# coding: utf-8

"""
    NetHSM
    All endpoints expect exactly the specified JSON. Additional properties will cause a Bad Request Error (400). All HTTP errors contain a JSON structure with an explanation of type string. All [base64](https://tools.ietf.org/html/rfc4648#section-4) encoded values are Big Endian.   # noqa: E501
    The version of the OpenAPI document: v1
    Generated by: https://github.com/openapi-json-schema-tools/openapi-json-schema-generator
"""

import copy
from http import client as http_client
import logging
import multiprocessing
import sys
import typing
import typing_extensions

import urllib3

from nethsm.client import exceptions
from nethsm.client import security_schemes
from nethsm.client.components.security_schemes import security_scheme_basic
from nethsm.client.servers import server_0

# security scheme key identifier to security scheme instance
SecuritySchemeInfo = typing.TypedDict(
    'SecuritySchemeInfo',
    {
        "basic": security_scheme_basic.Basic,
    },
    total=False
)


class SecurityIndexInfoRequired(typing.TypedDict):
    security: int

SecurityIndexInfoOptional = typing.TypedDict(
    'SecurityIndexInfoOptional',
    {
        "paths//config/backup-passphrase/put/security": typing.Literal[0],
        "paths//config/logging/get/security": typing.Literal[0],
        "paths//config/logging/put/security": typing.Literal[0],
        "paths//config/network/get/security": typing.Literal[0],
        "paths//config/network/put/security": typing.Literal[0],
        "paths//config/time/get/security": typing.Literal[0],
        "paths//config/time/put/security": typing.Literal[0],
        "paths//config/tls/cert.pem/get/security": typing.Literal[0],
        "paths//config/tls/cert.pem/put/security": typing.Literal[0],
        "paths//config/tls/csr.pem/post/security": typing.Literal[0],
        "paths//config/tls/generate/post/security": typing.Literal[0],
        "paths//config/tls/public.pem/get/security": typing.Literal[0],
        "paths//config/unattended-boot/get/security": typing.Literal[0],
        "paths//config/unattended-boot/put/security": typing.Literal[0],
        "paths//config/unlock-passphrase/put/security": typing.Literal[0],
        "paths//keys/get/security": typing.Literal[0],
        "paths//keys/post/security": typing.Literal[0],
        "paths//keys/generate/post/security": typing.Literal[0],
        "paths//keys/{KeyID}/delete/security": typing.Literal[0],
        "paths//keys/{KeyID}/get/security": typing.Literal[0],
        "paths//keys/{KeyID}/put/security": typing.Literal[0],
        "paths//keys/{KeyID}/cert/delete/security": typing.Literal[0],
        "paths//keys/{KeyID}/cert/get/security": typing.Literal[0],
        "paths//keys/{KeyID}/cert/put/security": typing.Literal[0],
        "paths//keys/{KeyID}/csr.pem/post/security": typing.Literal[0],
        "paths//keys/{KeyID}/decrypt/post/security": typing.Literal[0],
        "paths//keys/{KeyID}/encrypt/post/security": typing.Literal[0],
        "paths//keys/{KeyID}/public.pem/get/security": typing.Literal[0],
        "paths//keys/{KeyID}/restrictions/tags/{Tag}/delete/security": typing.Literal[0],
        "paths//keys/{KeyID}/restrictions/tags/{Tag}/put/security": typing.Literal[0],
        "paths//keys/{KeyID}/sign/post/security": typing.Literal[0],
        "paths//lock/post/security": typing.Literal[0],
        "paths//metrics/get/security": typing.Literal[0],
        "paths//random/post/security": typing.Literal[0],
        "paths//system/backup/post/security": typing.Literal[0],
        "paths//system/cancel-update/post/security": typing.Literal[0],
        "paths//system/commit-update/post/security": typing.Literal[0],
        "paths//system/factory-reset/post/security": typing.Literal[0],
        "paths//system/info/get/security": typing.Literal[0],
        "paths//system/reboot/post/security": typing.Literal[0],
        "paths//system/shutdown/post/security": typing.Literal[0],
        "paths//system/update/post/security": typing.Literal[0],
        "paths//users/get/security": typing.Literal[0],
        "paths//users/post/security": typing.Literal[0],
        "paths//users/{UserID}/delete/security": typing.Literal[0],
        "paths//users/{UserID}/get/security": typing.Literal[0],
        "paths//users/{UserID}/put/security": typing.Literal[0],
        "paths//users/{UserID}/passphrase/post/security": typing.Literal[0],
        "paths//users/{UserID}/tags/get/security": typing.Literal[0],
        "paths//users/{UserID}/tags/{Tag}/delete/security": typing.Literal[0],
        "paths//users/{UserID}/tags/{Tag}/put/security": typing.Literal[0],
    },
    total=False
)


class SecurityIndexInfo(SecurityIndexInfoRequired, SecurityIndexInfoOptional):
    """
    the default security_index to use at each openapi document json path
    the fallback value is stored in the 'security' key
    """

# the server to use at each openapi document json path
ServerInfo = typing.TypedDict(
    'ServerInfo',
    {
        'servers/0': server_0.Server0,
    },
    total=False
)


class ServerIndexInfoRequired(typing.TypedDict):
    servers: typing.Literal[0]

ServerIndexInfoOptional = typing.TypedDict(
    'ServerIndexInfoOptional',
    {
    },
    total=False
)


class ServerIndexInfo(ServerIndexInfoRequired, ServerIndexInfoOptional):
    """
    the default server_index to use at each openapi document json path
    the fallback value is stored in the 'servers' key
    """


class ApiConfiguration(object):
    """NOTE: This class is auto generated by OpenAPI JSON Schema Generator

    Ref: https://github.com/openapi-json-schema-tools/openapi-json-schema-generator
    Do not edit the class manually.

    :param security_scheme_info: the security scheme auth info that can be used when calling endpoints
      The key is a string that identifies the component security scheme that one is adding auth info for
      The value is an instance of the component security scheme class for that security scheme
      See the SecuritySchemeInfo TypedDict definition
    :param security_index_info: path to security_index information
    :param server_info: the servers that can be used to make endpoint calls
    :param server_index_info: index to servers configuration
    """

    def __init__(
        self,
        security_scheme_info: typing.Optional[SecuritySchemeInfo] = None,
        security_index_info: typing.Optional[SecurityIndexInfo] = None,
        server_info: typing.Optional[ServerInfo] = None,
        server_index_info: typing.Optional[ServerIndexInfo] = None,
    ):
        """Constructor
        """
        # Authentication Settings
        self.security_scheme_info: SecuritySchemeInfo = security_scheme_info or SecuritySchemeInfo()
        self.security_index_info: SecurityIndexInfo = security_index_info or {'security': 0}
        # Server Info
        self.server_info: ServerInfo = server_info or {
            'servers/0': server_0.Server0(),
        }
        self.server_index_info: ServerIndexInfo = server_index_info or {'servers': 0}
        self.logger = {}
        """Logging Settings
        """
        self.logger["package_logger"] = logging.getLogger("nethsm.client")
        self.logger["urllib3_logger"] = logging.getLogger("urllib3")
        self.logger_format = '%(asctime)s %(levelname)s %(message)s'
        """Log format
        """
        self.logger_stream_handler = None
        """Log stream handler
        """
        self.logger_file_handler = None
        """Log file handler
        """
        self.logger_file = None
        """Debug file location
        """
        self.debug = False
        """Debug switch
        """

        self.verify_ssl = True
        """SSL/TLS verification
           Set this to false to skip verifying SSL certificate when calling API
           from https server.
        """
        self.ssl_ca_cert = None
        """Set this to customize the certificate file to verify the peer.
        """
        self.cert_file = None
        """client certificate file
        """
        self.key_file = None
        """client key file
        """
        self.assert_hostname = None
        """Set this to True/False to enable/disable SSL hostname verification.
        """

        self.connection_pool_maxsize = multiprocessing.cpu_count() * 5
        """urllib3 connection pool's maximum number of connections saved
           per pool. urllib3 uses 1 connection as default value, but this is
           not the best value when you are making a lot of possibly parallel
           requests to the same host, which is often the case here.
           cpu_count * 5 is used as default value to increase performance.
        """

        self.proxy = None
        """Proxy URL
        """
        self.proxy_headers = None
        """Proxy headers
        """
        self.safe_chars_for_path_param = ''
        """Safe chars for path_param
        """
        self.retries = None
        """Adding retries to override urllib3 default value 3
        """
        # Enable client side validation
        self.client_side_validation = True

        # Options to pass down to the underlying urllib3 socket
        self.socket_options = None

    def __deepcopy__(self, memo):
        cls = self.__class__
        result = cls.__new__(cls)
        memo[id(self)] = result
        for k, v in self.__dict__.items():
            if k not in ('logger', 'logger_file_handler'):
                setattr(result, k, copy.deepcopy(v, memo))
        # shallow copy of loggers
        result.logger = copy.copy(self.logger)
        # use setters to configure loggers
        result.logger_file = self.logger_file
        result.debug = self.debug
        return result

    @property
    def logger_file(self):
        """The logger file.

        If the logger_file is None, then add stream handler and remove file
        handler. Otherwise, add file handler and remove stream handler.

        :param value: The logger_file path.
        :type: str
        """
        return self.__logger_file

    @logger_file.setter
    def logger_file(self, value):
        """The logger file.

        If the logger_file is None, then add stream handler and remove file
        handler. Otherwise, add file handler and remove stream handler.

        :param value: The logger_file path.
        :type: str
        """
        self.__logger_file = value
        if self.__logger_file:
            # If set logging file,
            # then add file handler and remove stream handler.
            self.logger_file_handler = logging.FileHandler(self.__logger_file)
            self.logger_file_handler.setFormatter(self.logger_formatter)
            for _, logger in self.logger.items():
                logger.addHandler(self.logger_file_handler)

    @property
    def debug(self):
        """Debug status

        :param value: The debug status, True or False.
        :type: bool
        """
        return self.__debug

    @debug.setter
    def debug(self, value):
        """Debug status

        :param value: The debug status, True or False.
        :type: bool
        """
        self.__debug = value
        if self.__debug:
            # if debug status is True, turn on debug logging
            for _, logger in self.logger.items():
                logger.setLevel(logging.DEBUG)
            # turn on http_client debug
            http_client.HTTPConnection.debuglevel = 1
        else:
            # if debug status is False, turn off debug logging,
            # setting log level to default `logging.WARNING`
            for _, logger in self.logger.items():
                logger.setLevel(logging.WARNING)
            # turn off http_client debug
            http_client.HTTPConnection.debuglevel = 0

    @property
    def logger_format(self):
        """The logger format.

        The logger_formatter will be updated when sets logger_format.

        :param value: The format string.
        :type: str
        """
        return self.__logger_format

    @logger_format.setter
    def logger_format(self, value):
        """The logger format.

        The logger_formatter will be updated when sets logger_format.

        :param value: The format string.
        :type: str
        """
        self.__logger_format = value
        self.logger_formatter = logging.Formatter(self.__logger_format)

    def to_debug_report(self):
        """Gets the essential information for debugging.

        :return: The report for debugging.
        """
        return "Python SDK Debug Report:\n"\
               "OS: {env}\n"\
               "Python Version: {pyversion}\n"\
               "Version of the API: v1\n"\
               "SDK Package Version: 1.0.0".\
               format(env=sys.platform, pyversion=sys.version)

    def get_server_url(
        self,
        key_prefix: typing.Literal[
            "servers",
        ],
        index: typing.Optional[int],
    ) -> str:
        """Gets host URL based on the index
        :param index: array index of the host settings
        :return: URL based on host settings
        """
        if index:
            used_index = index
        else:
            try:
                used_index = self.server_index_info[key_prefix]
            except KeyError:
                # fallback and use the default index
                used_index = self.server_index_info.get("servers", 0)
        server_info_key = typing.cast(
            typing.Literal[
                "servers/0",
            ],
            f"{key_prefix}/{used_index}"
        )
        try:
            server = self.server_info[server_info_key]
        except KeyError as ex:
            raise ex
        return server.url

    def get_security_requirement_object(
        self,
        key_prefix: typing.Literal[
            "security",
            "paths//config/backup-passphrase/put/security",
            "paths//config/logging/get/security",
            "paths//config/logging/put/security",
            "paths//config/network/get/security",
            "paths//config/network/put/security",
            "paths//config/time/get/security",
            "paths//config/time/put/security",
            "paths//config/tls/cert.pem/get/security",
            "paths//config/tls/cert.pem/put/security",
            "paths//config/tls/csr.pem/post/security",
            "paths//config/tls/generate/post/security",
            "paths//config/tls/public.pem/get/security",
            "paths//config/unattended-boot/get/security",
            "paths//config/unattended-boot/put/security",
            "paths//config/unlock-passphrase/put/security",
            "paths//keys/get/security",
            "paths//keys/post/security",
            "paths//keys/generate/post/security",
            "paths//keys/{KeyID}/delete/security",
            "paths//keys/{KeyID}/get/security",
            "paths//keys/{KeyID}/put/security",
            "paths//keys/{KeyID}/cert/delete/security",
            "paths//keys/{KeyID}/cert/get/security",
            "paths//keys/{KeyID}/cert/put/security",
            "paths//keys/{KeyID}/csr.pem/post/security",
            "paths//keys/{KeyID}/decrypt/post/security",
            "paths//keys/{KeyID}/encrypt/post/security",
            "paths//keys/{KeyID}/public.pem/get/security",
            "paths//keys/{KeyID}/restrictions/tags/{Tag}/delete/security",
            "paths//keys/{KeyID}/restrictions/tags/{Tag}/put/security",
            "paths//keys/{KeyID}/sign/post/security",
            "paths//lock/post/security",
            "paths//metrics/get/security",
            "paths//random/post/security",
            "paths//system/backup/post/security",
            "paths//system/cancel-update/post/security",
            "paths//system/commit-update/post/security",
            "paths//system/factory-reset/post/security",
            "paths//system/info/get/security",
            "paths//system/reboot/post/security",
            "paths//system/shutdown/post/security",
            "paths//system/update/post/security",
            "paths//users/get/security",
            "paths//users/post/security",
            "paths//users/{UserID}/delete/security",
            "paths//users/{UserID}/get/security",
            "paths//users/{UserID}/put/security",
            "paths//users/{UserID}/passphrase/post/security",
            "paths//users/{UserID}/tags/get/security",
            "paths//users/{UserID}/tags/{Tag}/delete/security",
            "paths//users/{UserID}/tags/{Tag}/put/security",
        ],
        security_requirement_objects: typing.List[security_schemes.SecurityRequirementObject],
        index: typing.Optional[int],
    ) -> security_schemes.SecurityRequirementObject:
        """Gets security_schemes.SecurityRequirementObject based on the index
        :param index: array index of the SecurityRequirementObject
        :return: the selected security_schemes.SecurityRequirementObject
        """
        if index:
            used_index = index
        else:
            try:
                used_index = self.security_index_info[key_prefix]
            except KeyError:
                # fallback and use the default index
                used_index = self.security_index_info.get("security", 0)
        return security_requirement_objects[used_index]



import datetime
import decimal
import io
import typing
import typing_extensions
import uuid

from nethsm.client import schemas, api_response

__all__ = [
    'decimal',
    'io',
    'typing',
    'typing_extensions',
    'uuid',
    'schemas',
    'api_response'
]


from __future__ import annotations
import datetime
import dataclasses
import io
import types
import typing
import uuid

import functools
import typing_extensions

from nethsm.client import exceptions
from nethsm.client.configurations import schema_configuration

from . import validation

_T_co = typing.TypeVar("_T_co", covariant=True)


class SequenceNotStr(typing.Protocol[_T_co]):
    """
    if a Protocol would define the interface of Sequence, this protocol
    would NOT allow str/bytes as their __contains__ methods are incompatible with the definition in Sequence
    methods from: https://docs.python.org/3/library/collections.abc.html#collections.abc.Collection
    """
    def __contains__(self, value: object, /) -> bool:
        raise NotImplementedError

    def __getitem__(self, index, /):
        raise NotImplementedError

    def __len__(self) -> int:
        raise NotImplementedError

    def __iter__(self) -> typing.Iterator[_T_co]:
        raise NotImplementedError

    def __reversed__(self, /) -> typing.Iterator[_T_co]:
        raise NotImplementedError

none_type_ = type(None)
T = typing.TypeVar('T', bound=typing.Mapping)
U = typing.TypeVar('U', bound=SequenceNotStr)
W = typing.TypeVar('W')


class SchemaTyped:
    additional_properties: typing.Type[Schema]
    all_of: typing.Tuple[typing.Type[Schema], ...]
    any_of: typing.Tuple[typing.Type[Schema], ...]
    discriminator: typing.Mapping[str, typing.Mapping[str, typing.Type[Schema]]]
    default: typing.Union[str, int, float, bool, None]
    enum_value_to_name: typing.Mapping[typing.Union[int, float, str, Bool, None], str]
    exclusive_maximum: typing.Union[int, float]
    exclusive_minimum: typing.Union[int, float]
    format: str
    inclusive_maximum: typing.Union[int, float]
    inclusive_minimum: typing.Union[int, float]
    items: typing.Type[Schema]
    max_items: int
    max_length: int
    max_properties: int
    min_items: int
    min_length: int
    min_properties: int
    multiple_of: typing.Union[int, float]
    not_: typing.Type[Schema]
    one_of: typing.Tuple[typing.Type[Schema], ...]
    pattern: validation.PatternInfo
    properties: typing.Mapping[str, typing.Type[Schema]]
    required: typing.FrozenSet[str]
    types: typing.FrozenSet[typing.Type]
    unique_items: bool


class FileIO(io.FileIO):
    """
    A class for storing files
    Note: this class is not immutable
    """

    def __new__(cls, arg: typing.Union[io.FileIO, io.BufferedReader]):
        if isinstance(arg, (io.FileIO, io.BufferedReader)):
            if arg.closed:
                raise exceptions.ApiValueError('Invalid file state; file is closed and must be open')
            arg.close()
            inst = super(FileIO, cls).__new__(cls, arg.name) # type: ignore
            super(FileIO, inst).__init__(arg.name)
            return inst
        raise exceptions.ApiValueError('FileIO must be passed arg which contains the open file')

    def __init__(self, arg: typing.Union[io.FileIO, io.BufferedReader]):
        """
        Needed for instantiation when passing in arguments of the above type
        """
        pass


class classproperty(typing.Generic[W]):
    def __init__(self, method: typing.Callable[..., W]):
        self.__method = method
        functools.update_wrapper(self, method) # type: ignore

    def __get__(self, obj, cls=None) -> W:
        if cls is None:
            cls = type(obj)
        return self.__method(cls)


class Bool:
    _instances: typing.Dict[typing.Tuple[type, bool], Bool] = {}
    """
    This class is needed to replace bool during validation processing
    json schema requires that 0 != False and 1 != True
    python implementation defines 0 == False and 1 == True
    To meet the json schema requirements, all bool instances are replaced with Bool singletons
    during validation only, and then bool values are returned from validation
    """

    def __new__(cls, arg_: bool, **kwargs):
        """
        Method that implements singleton
        cls base classes: BoolClass, NoneClass, str, decimal.Decimal
        The 3rd key is used in the tuple below for a corner case where an enum contains integer 1
        However 1.0  can also be ingested into that enum schema because 1.0 == 1 and
        Decimal('1.0') == Decimal('1')
        But if we omitted the 3rd value in the key, then Decimal('1.0') would be stored as Decimal('1')
        and json serializing that instance would be '1' rather than the expected '1.0'
        Adding the 3rd value, the str of arg_ ensures that 1.0 -> Decimal('1.0') which is serialized as 1.0
        """
        key = (cls, arg_)
        if key not in cls._instances:
            inst = super().__new__(cls)
            cls._instances[key] = inst
        return cls._instances[key]

    def __repr__(self):
        if bool(self):
            return f'<Bool: True>'
        return f'<Bool: False>'

    @classproperty
    def TRUE(cls):
        return cls(True) # type: ignore

    @classproperty
    def FALSE(cls):
        return cls(False) # type: ignore

    @functools.lru_cache()
    def __bool__(self) -> bool:
        for key, instance in self._instances.items():
            if self is instance:
                return bool(key[1])
        raise ValueError('Unable to find the boolean value of this instance')


def cast_to_allowed_types(
    arg: typing.Union[
        dict,
        validation.immutabledict,
        list,
        tuple,
        float,
        int,
        str,
        datetime.date,
        datetime.datetime,
        uuid.UUID,
        bool,
        None,
        bytes,
        io.FileIO,
        io.BufferedReader,
    ],
    from_server: bool,
    validated_path_to_schemas: typing.Dict[typing.Tuple[typing.Union[str, int], ...], typing.Set[typing.Union[str, int, float, bool, None, validation.immutabledict, tuple]]],
    path_to_item: typing.Tuple[typing.Union[str, int], ...],
    path_to_type: typing.Dict[typing.Tuple[typing.Union[str, int], ...], type]
) -> typing.Union[
    validation.immutabledict,
    tuple,
    float,
    int,
    str,
    bytes,
    Bool,
    None,
    FileIO
]:
    """
    Casts the input payload arg into the allowed types
    The input validated_path_to_schemas is mutated by running this function

    When from_server is False then
    - date/datetime is cast to str
    - int/float is cast to Decimal

    If a Schema instance is passed in it is converted back to a primitive instance because
    One may need to validate that data to the original Schema class AND additional different classes
    those additional classes will need to be added to the new manufactured class for that payload
    If the code didn't do this and kept the payload as a Schema instance it would fail to validate to other
    Schema classes and the code wouldn't be able to mfg a new class that includes all valid schemas
    TODO: store the validated schema classes in validation_metadata

    Args:
        arg: the payload
        from_server: whether this payload came from the server or not
        validated_path_to_schemas: a dict that stores the validated classes at any path location in the payload
    """
    type_error = exceptions.ApiTypeError(f"Invalid type. Required value type is str and passed type was {type(arg)} at {path_to_item}")
    if isinstance(arg, str):
        path_to_type[path_to_item] = str
        return str(arg)
    elif isinstance(arg, (dict, validation.immutabledict)):
        path_to_type[path_to_item] = validation.immutabledict
        return validation.immutabledict(
            {
                key: cast_to_allowed_types(
                    val,
                    from_server,
                    validated_path_to_schemas,
                    path_to_item + (key,),
                    path_to_type,
                )
                for key, val in arg.items()
            }
        )
    elif isinstance(arg, bool):
        """
        this check must come before isinstance(arg, (int, float))
        because isinstance(True, int) is True
        """
        path_to_type[path_to_item] = Bool
        if arg:
            return Bool.TRUE
        return Bool.FALSE
    elif isinstance(arg, int):
        path_to_type[path_to_item] = int
        return arg
    elif isinstance(arg, float):
        path_to_type[path_to_item] = float
        return arg
    elif isinstance(arg, (tuple, list)):
        path_to_type[path_to_item] = tuple
        return tuple(
            [
                cast_to_allowed_types(
                    item,
                    from_server,
                    validated_path_to_schemas,
                    path_to_item + (i,),
                    path_to_type,
                )
                for i, item in enumerate(arg)
            ]
        )
    elif arg is None:
        path_to_type[path_to_item] = type(None)
        return None
    elif isinstance(arg, (datetime.date, datetime.datetime)):
        path_to_type[path_to_item] = str
        if not from_server:
            return arg.isoformat()
        raise type_error
    elif isinstance(arg, uuid.UUID):
        path_to_type[path_to_item] = str
        if not from_server:
            return str(arg)
        raise type_error
    elif isinstance(arg, bytes):
        path_to_type[path_to_item] = bytes
        return bytes(arg)
    elif isinstance(arg, (io.FileIO, io.BufferedReader)):
        path_to_type[path_to_item] = FileIO
        return FileIO(arg)
    raise exceptions.ApiTypeError('Invalid type passed in got input={} type={}'.format(arg, type(arg)))


class SingletonMeta(type):
    """
    A singleton class for schemas
    Schemas are frozen classes that are never instantiated with init args
    All args come from defaults
    """
    _instances: typing.Dict[type, typing.Any] = {}

    def __call__(cls, *args, **kwargs):
        if cls not in cls._instances:
            cls._instances[cls] = super().__call__(*args, **kwargs)
        return cls._instances[cls]


class Schema(typing.Generic[T, U], validation.SchemaValidator, metaclass=SingletonMeta):

    @classmethod
    def __get_path_to_schemas(
        cls,
        arg,
        validation_metadata: validation.ValidationMetadata,
        path_to_type: typing.Dict[typing.Tuple[typing.Union[str, int], ...], typing.Type]
    ) -> typing.Dict[typing.Tuple[typing.Union[str, int], ...], typing.Type[Schema]]:
        """
        Run all validations in the json schema and return a dict of
        json schema to tuple of validated schemas
        """
        _path_to_schemas: validation.PathToSchemasType = {}
        if validation_metadata.validation_ran_earlier(cls):
            validation.add_deeper_validated_schemas(validation_metadata, _path_to_schemas)
        else:
            other_path_to_schemas = cls._validate(arg, validation_metadata=validation_metadata)
            validation.update(_path_to_schemas, other_path_to_schemas)
        # loop through it make a new class for each entry
        # do not modify the returned result because it is cached and we would be modifying the cached value
        path_to_schemas: typing.Dict[typing.Tuple[typing.Union[str, int], ...], typing.Type[Schema]] = {}
        for path, schema_classes in _path_to_schemas.items():
            schema = typing.cast(typing.Type[Schema], tuple(schema_classes)[-1])
            path_to_schemas[path] = schema
        """
        For locations that validation did not check
        the code still needs to store type + schema information for instantiation
        All of those schemas will be UnsetAnyTypeSchema
        """
        missing_paths = path_to_type.keys() - path_to_schemas.keys()
        for missing_path in missing_paths:
            path_to_schemas[missing_path] = UnsetAnyTypeSchema

        return path_to_schemas

    @staticmethod
    def __get_items(
        arg: tuple,
        path_to_item: typing.Tuple[typing.Union[str, int], ...],
        path_to_schemas: typing.Dict[typing.Tuple[typing.Union[str, int], ...], typing.Type[Schema]]
    ):
        '''
        Schema __get_items
        '''
        cast_items = []

        for i, value in enumerate(arg):
            item_path_to_item = path_to_item + (i,)
            item_cls = path_to_schemas[item_path_to_item]
            new_value = item_cls._get_new_instance_without_conversion(
                value,
                item_path_to_item,
                path_to_schemas
            )
            cast_items.append(new_value)

        return tuple(cast_items)

    @staticmethod
    def __get_properties(
        arg: validation.immutabledict[str, typing.Any],
        path_to_item: typing.Tuple[typing.Union[str, int], ...],
        path_to_schemas: typing.Dict[typing.Tuple[typing.Union[str, int], ...], typing.Type[Schema]]
    ):
        """
        Schema __get_properties, this is how properties are set
        These values already passed validation
        """
        dict_items = {}

        for property_name_js, value in arg.items():
            property_path_to_item = path_to_item + (property_name_js,)
            property_cls = path_to_schemas[property_path_to_item]
            new_value = property_cls._get_new_instance_without_conversion(
                value,
                property_path_to_item,
                path_to_schemas
            )
            dict_items[property_name_js] = new_value

        return validation.immutabledict(dict_items)

    @classmethod
    def _get_new_instance_without_conversion(
        cls,
        arg: typing.Union[int, float, None, Bool, str, validation.immutabledict, tuple, FileIO, bytes],
        path_to_item: typing.Tuple[typing.Union[str, int], ...],
        path_to_schemas: typing.Dict[typing.Tuple[typing.Union[str, int], ...], typing.Type[Schema]]
    ):
        # We have a Dynamic class and we are making an instance of it
        if isinstance(arg, validation.immutabledict):
            used_arg = cls.__get_properties(arg, path_to_item, path_to_schemas)
        elif isinstance(arg, tuple):
            used_arg = cls.__get_items(arg, path_to_item, path_to_schemas)
        elif isinstance(arg, Bool):
            return bool(arg)
        else:
            """
            str, int, float, FileIO, bytes
            FileIO = openapi binary type and the user inputs a file
            bytes = openapi binary type and the user inputs bytes
            """
            return arg
        arg_type = type(arg)
        type_to_output_cls = cls.__get_type_to_output_cls()
        if type_to_output_cls is None:
            return used_arg
        if arg_type not in type_to_output_cls:
            return used_arg
        output_cls = type_to_output_cls[arg_type]
        if arg_type is tuple:
            inst = super(output_cls, output_cls).__new__(output_cls, used_arg) # type: ignore
            inst = typing.cast(U, inst)
            return inst
        assert issubclass(output_cls, validation.immutabledict)
        inst = super(output_cls, output_cls).__new__(output_cls, used_arg) # type: ignore
        inst = typing.cast(T, inst)
        return inst

    @typing.overload
    @classmethod
    def validate_base(
        cls,
        arg: None,
        configuration: typing.Optional[schema_configuration.SchemaConfiguration] = None
    ) -> None: ...

    @typing.overload
    @classmethod
    def validate_base(
        cls,
        arg: typing.Literal[True],
        configuration: typing.Optional[schema_configuration.SchemaConfiguration] = None
    ) -> typing.Literal[True]: ...

    @typing.overload
    @classmethod
    def validate_base(
        cls,
        arg: typing.Literal[False],
        configuration: typing.Optional[schema_configuration.SchemaConfiguration] = None
    ) -> typing.Literal[False]: ...

    @typing.overload
    @classmethod
    def validate_base(
        cls,
        arg: bool,
        configuration: typing.Optional[schema_configuration.SchemaConfiguration] = None
    ) -> bool: ...

    @typing.overload
    @classmethod
    def validate_base(
        cls,
        arg: int,
        configuration: typing.Optional[schema_configuration.SchemaConfiguration] = None
    ) -> int: ...

    @typing.overload
    @classmethod
    def validate_base(
        cls,
        arg: float,
        configuration: typing.Optional[schema_configuration.SchemaConfiguration] = None
    ) -> float: ...

    @typing.overload
    @classmethod
    def validate_base(
        cls,
        arg: typing.Union[str, datetime.date, datetime.datetime, uuid.UUID],
        configuration: typing.Optional[schema_configuration.SchemaConfiguration] = None
    ) -> str: ...

    @typing.overload
    @classmethod
    def validate_base(
        cls,
        arg: SequenceNotStr[INPUT_TYPES_ALL],
        configuration: typing.Optional[schema_configuration.SchemaConfiguration] = None
    ) -> U: ...

    @typing.overload
    @classmethod
    def validate_base(
        cls,
        arg: U,
        configuration: typing.Optional[schema_configuration.SchemaConfiguration] = None
    ) -> U: ...

    @typing.overload
    @classmethod
    def validate_base(
        cls,
        arg: typing.Mapping[str, object],  # object needed as value type for typeddict inputs
        configuration: typing.Optional[schema_configuration.SchemaConfiguration] = None
    ) -> T: ...

    @typing.overload
    @classmethod
    def validate_base(
        cls,
        arg: typing.Union[
            typing.Mapping[str, INPUT_TYPES_ALL],
            T
        ],
        configuration: typing.Optional[schema_configuration.SchemaConfiguration] = None
    ) -> T: ...

    @typing.overload
    @classmethod
    def validate_base(
        cls,
        arg: typing.Union[io.FileIO, io.BufferedReader],
        configuration: typing.Optional[schema_configuration.SchemaConfiguration] = None
    ) -> FileIO: ...

    @typing.overload
    @classmethod
    def validate_base(
        cls,
        arg: bytes,
        configuration: typing.Optional[schema_configuration.SchemaConfiguration] = None
    ) -> bytes: ...

    @classmethod
    def validate_base(
        cls,
        arg,
        configuration: typing.Optional[schema_configuration.SchemaConfiguration] = None,
    ):
        """
        Schema validate_base

        Args:
            arg (int/float/str/list/tuple/dict/validation.immutabledict/bool/None): the value
            configuration: contains the schema_configuration.SchemaConfiguration that enables json schema validation keywords
                like minItems, minLength etc
        """
        if isinstance(arg, (tuple, validation.immutabledict)):
            type_to_output_cls = cls.__get_type_to_output_cls()
            if type_to_output_cls is not None:
                for output_cls in type_to_output_cls.values():
                    if isinstance(arg, output_cls):
                        # U + T use case, don't run validations twice
                        return arg

        from_server = False
        validated_path_to_schemas: typing.Dict[
            typing.Tuple[typing.Union[str, int], ...],
            typing.Set[typing.Union[str, int, float, bool, None, validation.immutabledict, tuple]]
        ] = {}
        path_to_type: typing.Dict[typing.Tuple[typing.Union[str, int], ...], type] = {}
        cast_arg = cast_to_allowed_types(
            arg, from_server, validated_path_to_schemas, ('args[0]',), path_to_type)
        validation_metadata = validation.ValidationMetadata(
            path_to_item=('args[0]',),
            configuration=configuration or schema_configuration.SchemaConfiguration(),
            validated_path_to_schemas=validation.immutabledict(validated_path_to_schemas)
        )
        path_to_schemas = cls.__get_path_to_schemas(cast_arg, validation_metadata, path_to_type)
        return cls._get_new_instance_without_conversion(
            cast_arg,
            validation_metadata.path_to_item,
            path_to_schemas,
        )

    @classmethod
    def __get_type_to_output_cls(cls) -> typing.Optional[typing.Mapping[type, type]]:
        type_to_output_cls = getattr(cls(), 'type_to_output_cls', None)
        type_to_output_cls = typing.cast(typing.Optional[typing.Mapping[type, type]], type_to_output_cls)
        return type_to_output_cls


def get_class(
    item_cls: typing.Union[types.FunctionType, staticmethod, typing.Type[Schema]],
    local_namespace: typing.Optional[dict] = None
) -> typing.Type[Schema]:
    if isinstance(item_cls, typing._GenericAlias): # type: ignore
        # petstore_api.schemas.StrSchema[~U] -> petstore_api.schemas.StrSchema
        origin_cls = typing.get_origin(item_cls)
        if origin_cls is None:
            raise ValueError('origin class must not be None')
        return origin_cls
    elif isinstance(item_cls, types.FunctionType):
        # referenced schema
        return item_cls()
    elif isinstance(item_cls, staticmethod):
        # referenced schema
        return item_cls.__func__()
    elif isinstance(item_cls, type):
        return item_cls
    elif isinstance(item_cls, typing.ForwardRef):
        return item_cls._evaluate(None, local_namespace)
    raise ValueError('invalid class value passed in')


@dataclasses.dataclass(frozen=True)
class AnyTypeSchema(Schema[T, U]):
    # Python representation of a schema defined as true or {}

    @typing.overload
    @classmethod
    def validate(
        cls,
        arg: None,
        configuration: typing.Optional[schema_configuration.SchemaConfiguration] = None
    ) -> None: ...

    @typing.overload
    @classmethod
    def validate(
        cls,
        arg: typing.Literal[True],
        configuration: typing.Optional[schema_configuration.SchemaConfiguration] = None
    ) -> typing.Literal[True]: ...

    @typing.overload
    @classmethod
    def validate(
        cls,
        arg: typing.Literal[False],
        configuration: typing.Optional[schema_configuration.SchemaConfiguration] = None
    ) -> typing.Literal[False]: ...

    @typing.overload
    @classmethod
    def validate(
        cls,
        arg: bool,
        configuration: typing.Optional[schema_configuration.SchemaConfiguration] = None
    ) -> bool: ...

    @typing.overload
    @classmethod
    def validate(
        cls,
        arg: int,
        configuration: typing.Optional[schema_configuration.SchemaConfiguration] = None
    ) -> int: ...

    @typing.overload
    @classmethod
    def validate(
        cls,
        arg: float,
        configuration: typing.Optional[schema_configuration.SchemaConfiguration] = None
    ) -> float: ...

    @typing.overload
    @classmethod
    def validate(
        cls,
        arg: typing.Union[str, datetime.date, datetime.datetime, uuid.UUID],
        configuration: typing.Optional[schema_configuration.SchemaConfiguration] = None
    ) -> str: ...

    @typing.overload
    @classmethod
    def validate(
        cls,
        arg: SequenceNotStr[INPUT_TYPES_ALL],
        configuration: typing.Optional[schema_configuration.SchemaConfiguration] = None
    ) -> U: ...

    @typing.overload
    @classmethod
    def validate(
        cls,
        arg: U,
        configuration: typing.Optional[schema_configuration.SchemaConfiguration] = None
    ) -> U: ...

    @typing.overload
    @classmethod
    def validate(
        cls,
        arg: typing.Union[
            typing.Mapping[str, INPUT_TYPES_ALL],
            T
        ],
        configuration: typing.Optional[schema_configuration.SchemaConfiguration] = None
    ) -> T: ...

    @typing.overload
    @classmethod
    def validate(
        cls,
        arg: typing.Union[io.FileIO, io.BufferedReader],
        configuration: typing.Optional[schema_configuration.SchemaConfiguration] = None
    ) -> FileIO: ...

    @typing.overload
    @classmethod
    def validate(
        cls,
        arg: bytes,
        configuration: typing.Optional[schema_configuration.SchemaConfiguration] = None
    ) -> bytes: ...

    @classmethod
    def validate(
        cls,
        arg,
        configuration: typing.Optional[schema_configuration.SchemaConfiguration] = None,
    ):
        return cls.validate_base(
            arg,
            configuration=configuration
        )

class UnsetAnyTypeSchema(AnyTypeSchema[T, U]):
    # Used when additionalProperties/items was not explicitly defined and a defining schema is needed
    pass

INPUT_TYPES_ALL = typing.Union[
    dict,
    validation.immutabledict,
    typing.Mapping[str, object],  # for TypedDict
    list,
    tuple,
    float,
    int,
    str,
    datetime.date,
    datetime.datetime,
    uuid.UUID,
    bool,
    None,
    bytes,
    io.FileIO,
    io.BufferedReader,
    FileIO
]


# coding: utf-8

"""
    NetHSM
    All endpoints expect exactly the specified JSON. Additional properties will cause a Bad Request Error (400). All HTTP errors contain a JSON structure with an explanation of type string. All [base64](https://tools.ietf.org/html/rfc4648#section-4) encoded values are Big Endian.   # noqa: E501
    The version of the OpenAPI document: v1
    Generated by: https://github.com/openapi-json-schema-tools/openapi-json-schema-generator
"""

import typing

import typing_extensions

from .schema import (
    get_class,
    none_type_,
    classproperty,
    Bool,
    FileIO,
    Schema,
    SingletonMeta,
    AnyTypeSchema,
    UnsetAnyTypeSchema,
    INPUT_TYPES_ALL
)

from .schemas import (
    ListSchema,
    NoneSchema,
    NumberSchema,
    IntSchema,
    Int32Schema,
    Int64Schema,
    Float32Schema,
    Float64Schema,
    StrSchema,
    UUIDSchema,
    DateSchema,
    DateTimeSchema,
    DecimalSchema,
    BytesSchema,
    FileSchema,
    BinarySchema,
    BoolSchema,
    NotAnyTypeSchema,
    OUTPUT_BASE_TYPES,
    DictSchema
)
from .validation import (
    PatternInfo,
    ValidationMetadata,
    immutabledict
)
from .format import (
    as_date,
    as_datetime,
    as_decimal,
    as_uuid
)

def typed_dict_to_instance(t_dict: typing_extensions._TypedDictMeta) -> typing.Mapping: # type: ignore
    res = {}
    for key, val in t_dict.__annotations__.items():
        if isinstance(val, typing._GenericAlias): # type: ignore
            # typing.Type[W] -> W
            val_cls = typing.get_args(val)[0]
            res[key] = val_cls
    return res

X = typing.TypeVar('X', bound=typing.Tuple)

def tuple_to_instance(tup: typing.Type[X]) -> X:
    res = []
    for arg in typing.get_args(tup):
        if isinstance(arg, typing._GenericAlias): # type: ignore
            # typing.Type[Schema] -> Schema
            arg_cls = typing.get_args(arg)[0]
            res.append(arg_cls)
    return tuple(res) # type: ignore


class Unset:
    """
    An instance of this class is set as the default value for object type(dict) properties that are optional
    When a property has an unset value, that property will not be assigned in the dict
    """
    pass

unset: Unset = Unset()

def key_unknown_error_msg(key: str) -> str:
    return (f"Invalid key. The key {key} is not a known key in this payload. "
        "If this key is an additional property, use get_additional_property_"
    )

def raise_if_key_known(
    key: str,
    required_keys: typing.FrozenSet[str],
    optional_keys: typing.FrozenSet[str]
):
    if key in required_keys or key in optional_keys:
        raise ValueError(f"The key {key} is a known property, use get_property to access its value")

__all__ = [
    'get_class',
    'none_type_',
    'classproperty',
    'Bool',
    'FileIO',
    'Schema',
    'SingletonMeta',
    'AnyTypeSchema',
    'UnsetAnyTypeSchema',
    'INPUT_TYPES_ALL',
    'ListSchema',
    'NoneSchema',
    'NumberSchema',
    'IntSchema',
    'Int32Schema',
    'Int64Schema',
    'Float32Schema',
    'Float64Schema',
    'StrSchema',
    'UUIDSchema',
    'DateSchema',
    'DateTimeSchema',
    'DecimalSchema',
    'BytesSchema',
    'FileSchema',
    'BinarySchema',
    'BoolSchema',
    'NotAnyTypeSchema',
    'OUTPUT_BASE_TYPES',
    'DictSchema',
    'PatternInfo',
    'ValidationMetadata',
    'immutabledict',
    'as_date',
    'as_datetime',
    'as_decimal',
    'as_uuid',
    'typed_dict_to_instance',
    'tuple_to_instance',
    'Unset',
    'unset',
    'key_unknown_error_msg',
    'raise_if_key_known'
]


# coding: utf-8

"""
    Generated by: https://github.com/openapi-json-schema-tools/openapi-json-schema-generator
"""

from nethsm.client.shared_imports.header_imports import *  # pyright: ignore [reportWildcardImportFromLibrary]

from . import schema


class Parameter1(api_client.QueryParameter):
    name = "systemTime"
    style = api_client.ParameterStyle.FORM
    schema: typing_extensions.TypeAlias = schema.Schema
    explode = True



import typing
import typing_extensions

from nethsm.client.apis.paths.config_backup_passphrase import ConfigBackupPassphrase
from nethsm.client.apis.paths.config_logging import ConfigLogging
from nethsm.client.apis.paths.config_network import ConfigNetwork
from nethsm.client.apis.paths.config_time import ConfigTime
from nethsm.client.apis.paths.config_tls_cert_pem import ConfigTlsCertPem
from nethsm.client.apis.paths.config_tls_csr_pem import ConfigTlsCsrPem
from nethsm.client.apis.paths.config_tls_generate import ConfigTlsGenerate
from nethsm.client.apis.paths.config_tls_public_pem import ConfigTlsPublicPem
from nethsm.client.apis.paths.config_unattended_boot import ConfigUnattendedBoot
from nethsm.client.apis.paths.config_unlock_passphrase import ConfigUnlockPassphrase
from nethsm.client.apis.paths.health_alive import HealthAlive
from nethsm.client.apis.paths.health_ready import HealthReady
from nethsm.client.apis.paths.health_state import HealthState
from nethsm.client.apis.paths.info import Info
from nethsm.client.apis.paths.keys import Keys
from nethsm.client.apis.paths.keys_generate import KeysGenerate
from nethsm.client.apis.paths.keys_key_id import KeysKeyID
from nethsm.client.apis.paths.keys_key_id_cert import KeysKeyIDCert
from nethsm.client.apis.paths.keys_key_id_csr_pem import KeysKeyIDCsrPem
from nethsm.client.apis.paths.keys_key_id_decrypt import KeysKeyIDDecrypt
from nethsm.client.apis.paths.keys_key_id_encrypt import KeysKeyIDEncrypt
from nethsm.client.apis.paths.keys_key_id_public_pem import KeysKeyIDPublicPem
from nethsm.client.apis.paths.keys_key_id_restrictions_tags_tag import KeysKeyIDRestrictionsTagsTag
from nethsm.client.apis.paths.keys_key_id_sign import KeysKeyIDSign
from nethsm.client.apis.paths.lock import Lock
from nethsm.client.apis.paths.metrics import Metrics
from nethsm.client.apis.paths.provision import Provision
from nethsm.client.apis.paths.random import Random
from nethsm.client.apis.paths.system_backup import SystemBackup
from nethsm.client.apis.paths.system_cancel_update import SystemCancelUpdate
from nethsm.client.apis.paths.system_commit_update import SystemCommitUpdate
from nethsm.client.apis.paths.system_factory_reset import SystemFactoryReset
from nethsm.client.apis.paths.system_info import SystemInfo
from nethsm.client.apis.paths.system_reboot import SystemReboot
from nethsm.client.apis.paths.system_restore import SystemRestore
from nethsm.client.apis.paths.system_shutdown import SystemShutdown
from nethsm.client.apis.paths.system_update import SystemUpdate
from nethsm.client.apis.paths.unlock import Unlock
from nethsm.client.apis.paths.users import Users
from nethsm.client.apis.paths.users_user_id import UsersUserID
from nethsm.client.apis.paths.users_user_id_passphrase import UsersUserIDPassphrase
from nethsm.client.apis.paths.users_user_id_tags import UsersUserIDTags
from nethsm.client.apis.paths.users_user_id_tags_tag import UsersUserIDTagsTag

PathToApi = typing.TypedDict(
    'PathToApi',
    {
    "/config/backup-passphrase": typing.Type[ConfigBackupPassphrase],
    "/config/logging": typing.Type[ConfigLogging],
    "/config/network": typing.Type[ConfigNetwork],
    "/config/time": typing.Type[ConfigTime],
    "/config/tls/cert.pem": typing.Type[ConfigTlsCertPem],
    "/config/tls/csr.pem": typing.Type[ConfigTlsCsrPem],
    "/config/tls/generate": typing.Type[ConfigTlsGenerate],
    "/config/tls/public.pem": typing.Type[ConfigTlsPublicPem],
    "/config/unattended-boot": typing.Type[ConfigUnattendedBoot],
    "/config/unlock-passphrase": typing.Type[ConfigUnlockPassphrase],
    "/health/alive": typing.Type[HealthAlive],
    "/health/ready": typing.Type[HealthReady],
    "/health/state": typing.Type[HealthState],
    "/info": typing.Type[Info],
    "/keys": typing.Type[Keys],
    "/keys/generate": typing.Type[KeysGenerate],
    "/keys/{KeyID}": typing.Type[KeysKeyID],
    "/keys/{KeyID}/cert": typing.Type[KeysKeyIDCert],
    "/keys/{KeyID}/csr.pem": typing.Type[KeysKeyIDCsrPem],
    "/keys/{KeyID}/decrypt": typing.Type[KeysKeyIDDecrypt],
    "/keys/{KeyID}/encrypt": typing.Type[KeysKeyIDEncrypt],
    "/keys/{KeyID}/public.pem": typing.Type[KeysKeyIDPublicPem],
    "/keys/{KeyID}/restrictions/tags/{Tag}": typing.Type[KeysKeyIDRestrictionsTagsTag],
    "/keys/{KeyID}/sign": typing.Type[KeysKeyIDSign],
    "/lock": typing.Type[Lock],
    "/metrics": typing.Type[Metrics],
    "/provision": typing.Type[Provision],
    "/random": typing.Type[Random],
    "/system/backup": typing.Type[SystemBackup],
    "/system/cancel-update": typing.Type[SystemCancelUpdate],
    "/system/commit-update": typing.Type[SystemCommitUpdate],
    "/system/factory-reset": typing.Type[SystemFactoryReset],
    "/system/info": typing.Type[SystemInfo],
    "/system/reboot": typing.Type[SystemReboot],
    "/system/restore": typing.Type[SystemRestore],
    "/system/shutdown": typing.Type[SystemShutdown],
    "/system/update": typing.Type[SystemUpdate],
    "/unlock": typing.Type[Unlock],
    "/users": typing.Type[Users],
    "/users/{UserID}": typing.Type[UsersUserID],
    "/users/{UserID}/passphrase": typing.Type[UsersUserIDPassphrase],
    "/users/{UserID}/tags": typing.Type[UsersUserIDTags],
    "/users/{UserID}/tags/{Tag}": typing.Type[UsersUserIDTagsTag],
    }
)

path_to_api = PathToApi(
    {
    "/config/backup-passphrase": ConfigBackupPassphrase,
    "/config/logging": ConfigLogging,
    "/config/network": ConfigNetwork,
    "/config/time": ConfigTime,
    "/config/tls/cert.pem": ConfigTlsCertPem,
    "/config/tls/csr.pem": ConfigTlsCsrPem,
    "/config/tls/generate": ConfigTlsGenerate,
    "/config/tls/public.pem": ConfigTlsPublicPem,
    "/config/unattended-boot": ConfigUnattendedBoot,
    "/config/unlock-passphrase": ConfigUnlockPassphrase,
    "/health/alive": HealthAlive,
    "/health/ready": HealthReady,
    "/health/state": HealthState,
    "/info": Info,
    "/keys": Keys,
    "/keys/generate": KeysGenerate,
    "/keys/{KeyID}": KeysKeyID,
    "/keys/{KeyID}/cert": KeysKeyIDCert,
    "/keys/{KeyID}/csr.pem": KeysKeyIDCsrPem,
    "/keys/{KeyID}/decrypt": KeysKeyIDDecrypt,
    "/keys/{KeyID}/encrypt": KeysKeyIDEncrypt,
    "/keys/{KeyID}/public.pem": KeysKeyIDPublicPem,
    "/keys/{KeyID}/restrictions/tags/{Tag}": KeysKeyIDRestrictionsTagsTag,
    "/keys/{KeyID}/sign": KeysKeyIDSign,
    "/lock": Lock,
    "/metrics": Metrics,
    "/provision": Provision,
    "/random": Random,
    "/system/backup": SystemBackup,
    "/system/cancel-update": SystemCancelUpdate,
    "/system/commit-update": SystemCommitUpdate,
    "/system/factory-reset": SystemFactoryReset,
    "/system/info": SystemInfo,
    "/system/reboot": SystemReboot,
    "/system/restore": SystemRestore,
    "/system/shutdown": SystemShutdown,
    "/system/update": SystemUpdate,
    "/unlock": Unlock,
    "/users": Users,
    "/users/{UserID}": UsersUserID,
    "/users/{UserID}/passphrase": UsersUserIDPassphrase,
    "/users/{UserID}/tags": UsersUserIDTags,
    "/users/{UserID}/tags/{Tag}": UsersUserIDTagsTag,
    }
)
